; *********************************************************
; Long Period Astable timer
; For PIC 12F629 / 12F675  (will also run on 12F685 with minimal changes)
;
; Pete Griffiths, January 2009
; Version 1.2
;
; To assemble this code use MPLAB IDE
;
; From the MPLAB menu choose "Config - Select Device" and set the Device: to either 12F629 or 12F675 
; depending on which PIC you are going to use.
;
; Next from the menu select
; "File - Open" and load LongDelayF5.asm
;
; then
;
; "Project - Quickbuild" LongDelayF5.asm
;
; If there are any errors reported during the quickbuild you must correct
; them and retry the quickbuild until it is successful.
;
; You will also need to open LongDelayTimes.inc and edit the delays
; to suit your particular application.  You should do this before 
; re-assembling and remember to save the file.
;
;
; This is free software for non-commercial use only.
;
; This software and accompanying information is are provided "as is",d
; and without warranty of any kind, either express or implied.
;
; Before use you should read information in this file and at 
; http://picprojects.org.uk/projects/ldt/ldt.htm
; 
; If both the Q and notQ outputs go high and remain high after reset then
; the calibration word at program memory 0x3FF has been erased or lost.
; You can recover the calibration word using a Microchip PICkit2 programmer or,
; if you don't have one see http://picprojects.org.uk/projects/recal/recal.htm
;
;
; ********************************************************************************
; 
;	              ___ ___
;  	       Vdd   -|1 ^ 8|- Vss
;        Reset edge select -|2   7|- Q out
;        Hold (active low) -|3   6|- notQ out
;  MCLR tie to Vdd via 1K0 -|4   5|- reset (edge triggered)
;                           
;
; All inputs use the PIC internal weak-pull up so no external pull resistor
; is required.
;
; Reset edge select.  Connect to Vss for falling edge, leave open for rising edge
;
; MCLR must be tied to Vdd using a 1K resistor.
; ********************************************************************************
 

  IFDEF  __12F675
                #include "p12f675.inc"
    ENDIF

  IFDEF  __12F629
                #include "p12f629.inc"                
  ENDIF    


          __CONFIG       _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_ON & _CPD_OFF
          
          errorlevel -302     ; suppress banksel warning messages
          errorlevel -311     ; suppress HIGH operator warning messages

	cblock 0x20
 	 lo	; tick timer (3 bytes)
	 mid	; counts nominal 1 second intervals
	 hi	; 
	 tablePtrLo	; lookup table pointer Lo Byte
	 tablePtrHi	; lookup table pointer Hi byte
	 delaylo	; seconds counter low
	 delayhi	; and high byte pair
	 copyout	; working output register (copied to GPIO port)
	endc

#define   bank0     bcf       STATUS,RP0          ; Sel Bank 0
#define   bank1     bsf       STATUS,RP0          ; Sel Bank 1

; Define I/O pin functions
#define	Qout	0
#define	notQout	1
#define	rset	2 ; This must be GPIO2 as it use the edge triggered interrupt
#define	hold	4
#define	edgeSelect	5




	org 0x000
	nop
	btfsc	INTCON, INTE
	goto	badOsccal
	goto	start
	
	org 0x004
	;---------------------------------------
	; Interrupt handler
	;---------------------------------------

	btfsc	INTCON,INTF
	call	resetOut

	btfsc	INTCON,T0IF
	call	tick
	
	retfie	; and return from interrupt
	;---------------------------------------


	;------------------------------------------------------------------------------
	; Initialisation code
	;------------------------------------------------------------------------------

start	movlw	(1<<Qout)
	movwf	copyout
	movwf	GPIO
	
	movlw     	0x07            
          	movwf	CMCON        	; Disable Comparator on GPIO pins
   	
	bank1

	
	IFDEF __12F675
	clrf	ANSEL	; Disable analogue inputs (12F675 / 683 only)
	ENDIF

          	movlw	~(1<<Qout | 1 <<notQout) ; Specifiy GPIO port direction
          	movwf	TRISIO         
     	
	bcf	OPTION_REG, INTEDG
          	bcf	OPTION_REG,NOT_GPPU 	; enable global weak pull-up.
	bcf	OPTION_REG, PSA	; prescaler to TMR0
	bcf	OPTION_REG, T0CS	; TMR0 clock soruce internal
	bcf	OPTION_REG, PS0	; Set TMR0 prescaler to 1:2	
	bcf	OPTION_REG, PS1
	bcf	OPTION_REG, PS2

	bsf	INTCON, INTE
	call	0x3FF	; get factory calibration byte
	movwf	OSCCAL	; write to OSCCAL register
	bcf	INTCON, INTE

	movlw	(1<<rset | 1<< hold | 1<< edgeSelect)
	movwf	WPU

	bank0

	call	resetOut	; initialise outputs and timers
	
	clrw		; set reset edge
	btfsc	GPIO, edgeSelect
	movlw	(1<<INTEDG)	
	bank1
	iorwf	OPTION_REG,F	; set GP2 interrupt edge
	bank0		; 0 -> falling edge | 1 -> rising edge

	; setup interrupts
	bcf	INTCON, INTF	; clear GP2 int flag
	bcf	INTCON, T0IF	; clear TMR0 interrupt flag bit
	bsf	INTCON, INTE	; enagle GP2 edge interrupts
	bsf	INTCON, T0IE	; enable TMR0 interrupts
	bsf	INTCON, GIE	; enable Global interrupts
		

idle	goto	idle

	;------------------------------------------------------------------------------
	; Tick timer function
	; Get a nominal 1 second tick from 1Mhz instruction clock
	;------------------------------------------------------------------------------
	; TMR0 prescaler divides 1Mhz instruction clock by 2
	; 1Mhz = 1,000,000 / 2 = 500,000
	; 8-bit timer divides the prescaled clock 256

period equ .500000


tick	bcf	INTCON, T0IF	; clear TMR0 interrupt flag

	btfss	GPIO, hold
	return

	movlw	-.1
	addwf	mid,F	; subtract 1 from mid
	skpc		; skip next if no underflow
	addwf	hi,F	; subtract 1 from high
	skpnc		; skip next if it did underflow
	return		
			; nominal 1 second has passed so we
			; add 500,000 - 256 back into the counter
reload	movlw	LOW period	;	
	addwf	lo,F	; remainder in lo counter byte is added back

	movlw	HIGH period-.256	
	skpnc		; if low byte overflowed 
	movlw	HIGH period	; increment mid byte by one
	addwf	mid,F

	movlw	UPPER period	; mid byte never overflows so we
	movwf	hi	; just reset the hi byte

	
	;------------------------------------
	; Delay and output
	;-----------------------------------

; State 1	Delay timer
delay	decfsz	delaylo,F	; decrement delay counter
	return		; return if lo byte not zero

	decf	delayhi,F	; This is a bit odd because we pre-decrement
	incfsz	delayhi,W	; the hi byte ahead of the lo byte underflowing
	return		; but we only need to know when the count
			; reahces zero so this works as required

changeOut	call	lookup	; get the next delay value from the table lookup
	movwf	delaylo
	call	lookup
	movwf	delayhi
	xorwf	delaylo,W
	skpz		; skip if delay pair value is zero
	goto	setOutput	; if delay read from table isn't zero, toggle output bits
	call	initTablePtr	; if we've reached the end of the table
	goto	changeOut	; reset table pointer and go round again



setOutput	comf	copyout,W	; toggle bits in copyout, save in W
	movwf	copyout	; write W back to copy out
	movwf	GPIO	; and to GPIO port
	return


resetOut	movlw	~(1<<Qout)	; This sets the copyout variable to a space state. 
	movwf	copyout	; 
	comf	copyout,W	; then changes it to a mark
	movwf	GPIO	; before writing it to the GPIO port
			; This is done to make the output 'reset' as soon
			; as possible after the interrupt without being.  
			; affected by the following code.
				

	bcf	INTCON,INTF	; clear timer interrupt flag

	call	initTablePtr	; initialise table pointer

	movlw	LOW period	; initialise timer counter
	movwf	lo
	movlw	HIGH period
	movwf	mid
	movlw	UPPER period
	movwf	hi

	goto	changeOut


	; If the OSCCAL word is missing the code will halt
	; with Q / notQ outputs both high
badOsccal	bank0		
	movlw	(1<<Qout | 1<<notQout)
	movwf	GPIO
stop	goto	stop


	;---------------------------------------------------
	; initialise table pointer base address to start
	; of time delay table
initTablePtr
	movlw	HIGH delay.base.addr
	movwf	tablePtrHi
	movlw	LOW delay.base.addr
	movwf	tablePtrLo
	return


	;--------------------------------------------------
	; LongDelayTimes.inc file is included here
	; macros used to simplify the time entries are also
	; defined below
	;--------------------------------------------------

	; Table lookup with post increment of table address pointer.

lookup	movfw	tablePtrHi
	movwf	PCLATH
	movfw	tablePtrLo
	incf	tablePtrLo,F
	skpnz
	incf	tablePtrHi,F
	movwf	PCL

	; Macro definition,  used in the LongDelayTimes.inc file to make creation of the delay
	; entries user friendly.
time	MACRO	seconds
	radix decimal
	if seconds == 0
	 error "Invalid delay time value entered in LongDelayTimes.inc file. "
	 error "Value is 0 and must be 1 <= Delay <=65535. Please correct and reassemble."
	endif

	if seconds >65535 
	 error "Invalid delay time value entered in LongDelayTimes.inc file. "
	 error "Value is >65535 and must be 1 <= Delay <=65535. Please correct and reassemble."
	endif

	retlw	LOW seconds
	retlw	HIGH seconds
	radix hex
	ENDM

loop	MACRO	
	retlw	0x00
	retlw	0x00
	ENDM

delay.base.addr
	#include	LongDelayTimes.inc

	loop		; indicates end of data
			; lookup pointer is reset to start of data
	



	end